@@ -1,6 +1,6 @@
 package swat;
 
-our $VERSION = '0.1.79';
+our $VERSION = '0.1.80';
 
 use base 'Exporter'; 
 
@@ -55,19 +55,25 @@ sub execute_with_retry {
 
 sub make_http_request {
 
-    return get_prop('http_response') if defined get_prop('http_response');
+    return if get_prop('response_done');
 
     my ($fh, $content_file) = tempfile( DIR => get_prop('test_root_dir') );
 
     if (get_prop('response') and @{get_prop('response')} ){
 
-        ok(1,'response is already set');
+        ok(1,'server response is spoofed');
 
         open F, ">", $content_file or die $!;
         print F ( join "\n", @{get_prop('response')});
         close F;
 
-        diag "response saved to $content_file";
+        open F, ">", "$content_file.stderr" or die $!;
+        close F;
+
+        open F, ">", "$content_file.hdr" or die $!;
+        close F;
+
+        diag "spoofed response saved to $content_file";
 
     }else{
 
@@ -78,9 +84,9 @@ sub make_http_request {
 
         $curl_cmd.=' -f'  unless ignore_http_err();
 
-        my $curl_runner = "$curl_cmd -i -o $content_file --stderr $content_file.stderr '$hostname$resource'";
+        my $curl_runner = "$curl_cmd -D $content_file.hdr -o $content_file --stderr $content_file.stderr '$hostname$resource'";
 
-        my $st = execute_with_retry("$curl_runner && test -f $content_file", get_prop('try_num'));
+        my $st = execute_with_retry("$curl_runner && test -f $content_file.hdr", get_prop('try_num'));
 
         if ($st) {
 
@@ -98,6 +104,7 @@ sub make_http_request {
 
             diag($curl_runner);
 
+            diag "stderr:";
             open CURL_ERR, "$content_file.stderr" or die $!;
             while  ( my $i = <CURL_ERR>){
                 chomp $i;
@@ -105,31 +112,59 @@ sub make_http_request {
             }
             close CURL_ERR;
 
-            open CURL_OUT, "$content_file" or die $!;
-            while  ( my $i = <CURL_OUT>){
+            diag "http headers:";
+            open CURL_HDR, "$content_file.hdr" or die $!;
+            while  ( my $i = <CURL_HDR>){
                 chomp $i;
                 diag($i);
             }
-            close CURL_OUT;
+            close CURL_HDR;
+
+            diag "http body:";
+            open CURL_RSP, "$content_file" or die $!;
+            while  ( my $i = <CURL_RSP>){
+                chomp $i;
+                diag($i);
+            }
+            close CURL_RSP;
+
             diag("can't continue here due to unsuccessfull http status code");
             exit(1);
         }
 
-        diag "response saved to $content_file";
+        diag "response http headers saved to $content_file.hdr";
+        diag "response body saved to $content_file";
 
     }
 
+
     open F, $content_file or die $!;
-    my $http_response = '';
-    $http_response.= $_ while <F>;
+    my $body_str = '';
+    $body_str.= $_ while <F>;
     close F;
-    set_prop( http_response => $http_response );
 
-    my $debug_bytes = get_prop('debug_bytes');
+    set_prop( body => $body_str );
 
-    diag `head -c $debug_bytes $content_file` if debug_mod2();
+    open F, "$content_file.hdr" or die $!;
+    my $headers_str = '';
+    $headers_str.= $_ while <F>;
+    close F;
+
+    set_prop( headers => $headers_str );
+
+    if (debug_mod12()){
+        my $debug_bytes = get_prop('debug_bytes');
+        my $bshort = substr( $body_str, 0, $debug_bytes );
+        if (length($bshort) < length($body_str)) {
+             diag("$bshort ... ( output truncated to $debug_bytes bytes )"); 
+        } else{
+             diag($body_str);
+        }
+    }
+
+
+    set_prop( response_done => 1 );
 
-    return get_prop('http_response');
 }
 
 sub header {
@@ -149,9 +184,6 @@ sub header {
     ok(1, "hostname: $hostname");
     ok(1, "resource: $resource");
     ok(1, "http method: $http_method");
-    if ( @{get_prop('response' )}){
-        ok(1, 'response is set, so we do not use curl')
-    }
     ok(1,"swat module: $swat_module");
     ok(1, "debug: $debug");
     ok(1, "try num: $try_num");
@@ -170,7 +202,11 @@ sub generate_asserts {
 
     dsl()->{match_l} = get_prop('match_l');
 
-    dsl()->{output} = make_http_request();
+    make_http_request();
+
+    dsl()->{output} = headers().body();
+
+    run_response_processor();
 
     eval {
         dsl()->validate($check_file);
@@ -1052,19 +1088,18 @@ We could write such a code:
 
 =head2 Process http responses 
 
-B<I< not implemented yet >>
-
-I<process_response(CODEREF)>
+I<set_response_processor(CODEREF)>
 
 Response processors are custom perl function to modify content returned from server I<before> invoking a validation process.
 Processor code should be I<defined> by calling a process_response function with parameter as reference to processor function:
 
 For example:
 
-       process_response( sub { 
-          my $body = shift;
+       set_response_processor( sub { 
+          my $headers   = shift; # original response, http headers, String
+          my $body      = shift; # original response, body, String
           $body=~s/hello/swat/;
-          return $s
+          return $body;          # modified response will be return value of sub {} 
         });
 
 What you should know about processor functions:
@@ -1144,7 +1179,8 @@ For example:
       }
     
       # processor function
-      my $body = shift;
+      my $headers   = shift;
+      my $body      = shift;
       use JSON;
       $hash = decode_json($body);
       return 'Foo.Bar.Baz :'.( $hash->{Foo}->{Bar}->{Baz} )."\n";
@@ -1158,7 +1194,8 @@ For example:
       </foo>
     
       # processor function
-      my $body = shift;
+      my $headers   = shift;
+      my $body      = shift;
       use XML:LibXML;
       my $doc = XML::LibXML->parse_string($body);
       return 'Foo.Bar.Baz :'.( $doc->find("string(/foo/bar/baz)")->string_value )."\n";
