@@ -22,6 +22,9 @@ our @EXPORT = qw{
 
     modify_resource
 
+    set_response_processor
+    run_response_processor
+
     hostname ignore_http_err
 
     project_root_dir
@@ -31,6 +34,11 @@ our @EXPORT = qw{
     resource resource_dir
 
     http_method
+
+    body
+
+    headers
+
 };
 
 our @stories = ();
@@ -102,6 +110,14 @@ sub http_method {
     get_prop('http_method');
 }
 
+sub body {
+    get_prop('body');
+}
+
+sub headers {
+    get_prop('headers');
+}
+
 
 
 sub context_populated {
@@ -229,6 +245,43 @@ sub modify_resource {
 
 }
 
+sub set_response_processor {
+
+    my $sub = shift;
+
+    Test::More::ok(1,"setting response processor") if debug_mod12();
+
+    set_prop( response_processor => $sub );
+
+}
+
+sub run_response_processor {
+
+    return unless get_prop( 'response_processor' );
+
+    my $body = body();
+
+    my $headers = headers();
+
+    Test::More::ok(1,"running response processor") if debug_mod12();
+
+    my $sub = get_prop( 'response_processor' );
+
+    my $retval = $sub->($headers,$body);
+
+    if (debug_mod12()){
+        my $debug_bytes = get_prop('debug_bytes');
+        my $rshort = substr( $retval, 0, $debug_bytes );
+        if (length($rshort) < length($retval)) {
+             Test::More::diag("modified response:\n$rshort ... ( output truncated to $debug_bytes bytes )");
+        } else{
+             Test::More::diag("modified response:\n$retval");
+        }
+    }
+
+    dsl()->{output} = $retval;
+}
+
 
 1;
 
